﻿using System.ComponentModel;
using System.IO;
using System.Xml;
using PerpetuumSoft.Framework.Serialization;
using PerpetuumSoft.Reporting.Components;
using PerpetuumSoft.Reporting.DOM;
using ReportSlotCustomized.LocalExtensions;

namespace ReportSlotCustomized
{
    public class CustomFileReportSlot : FileReportSlot
    {
        public CustomFileReportSlot(IContainer container) : base(container)
        {
        }

        public override Document LoadReport()
        {
            var report = base.LoadReport();
            report.AddFooterTo("dataBand1");
            
            return report;
        }

        public override void SaveReport(Document document)
        {
            document.RemoveFooterFrom("dataBand1");
            base.SaveReport(document);
        }
    }

    namespace LocalExtensions
    {
        internal static class DocumentExtensions
        {
            private static readonly string _footerXml = @"
                        <Item type='PerpetuumSoft.Reporting.DOM.Footer' Location='0;472.44094848632812' Name='footer1' Size='2480.3149606299212;177.16535949707031'>
                            <Controls type='PerpetuumSoft.Reporting.DOM.ReportControlCollection' >
                            <Item type='PerpetuumSoft.Reporting.DOM.TextBox' Location='944.88189697265625;0' Name='textBox5' Text='THIS IS THE COMMON FOOTER' Size='1417.3228454589844;177.16535949707031'>
                                <DataBindings type='PerpetuumSoft.Reporting.DOM.ReportDataBindingCollection' />
                            </Item>
                            </Controls>
                            <Aggregates type='PerpetuumSoft.Reporting.DOM.AggregateCollection' />
                            <DataBindings type='PerpetuumSoft.Reporting.DOM.ReportDataBindingCollection' />
                        </Item>";

            internal static void AddFooterTo(this Document report, string controlName)
            {
                var footer = CreateFooter();

                foreach (Page page in report.Pages) {
                    var dataBand = page.ControlByName(controlName) as DataBand;
                    if (dataBand != null)
                        dataBand.Controls.Add(footer);
                }
            }

            internal static void RemoveFooterFrom(this Document report, string controlName)
            {
                foreach (Page page in report.Pages) {
                    var dataBand = page.ControlByName(controlName) as DataBand;
                    if (dataBand == null)
                        continue;

                    var controls = new ReportControlCollection();
                    foreach (ReportControl control in dataBand.Controls) {
                        if (!(control is Footer)) {
                            controls.Add(control);
                        }
                    }
                    while (dataBand.Controls.Count > 0) {
                        dataBand.Controls.RemoveAt(0);
                    }
                    foreach (ReportControl control in controls) {
                        dataBand.Controls.Add(control);
                    }
                }
            }

            private static Footer CreateFooter()
            {
                object footerObj = null;
                using (var reader = new XmlTextReader(new StringReader(_footerXml))) {
                    XSerializationManager.GetSerializer(typeof(Footer)).Deserialize(ref footerObj, reader, typeof(Footer), new XSerializationManager());
                }
                return footerObj as Footer;
            }
        }
    }
}
